==> sapdb-source-7.3.00.48.tgz: V73_00_48/SAPDB_ORG/sys/src/en/ven38.cpp <==
/* @lastChanged: "1998-10-06  15:03"
 * @filename:    ven38.cpp
 * @purpose:     "COM_Interface"
 * @release:     7.2.0.0
 * @see:         ""
 *
 * @copyright:   "Copyright by SAP AG, 1998"


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

 */

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#ifndef USE_KGS
#define USE_KGS
#endif

#if defined(NMP)
#define RWSTD_NO_NAMESPACE  1
#include <sys/user.h>
#endif

#include "gsp00.h"
#include "heo01.h"
#include "heo02x.h"
#include "heo78.h"
#include "veo00c.h" //
#include "gco00.h"
#include "geo00d.h"
#include "hak341.h"
#include "geo38.h"
#include "hen38.h"

#include "heo52.h"
#include "vsp002c.h"
#include "heo46.h"
#include "geo60.h"
#include "heo60.h"
#include "geo50_0.h"
#include "gen500.h"
#include "heo38.h"

 /*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

#ifdef NO_INLINES
# define _INLINE
#else
# define _INLINE	inline
#endif

#define DCOM_EXCLUSIVE  KGS->DCOM_MEM.lExclusive
#define DCOM_HEAD       KGS->DCOM_MEM.lphead
#define DCOM_FREE       KGS->DCOM_MEM.lpfree

typedef BOOL (*DLLMAIN) (HANDLE hInstance,
                         DWORD  dwReason, 
                         char*  szModule);

typedef HRESULT (*DLLGETCLASSOBJECT) (REFCLSID rclsid, REFIID riid, LPVOID* ppv);

/*---------------------------------------------------------------------------*/

long sql38CoCreateInstance(CLSID *rclsid, tsp00_Addr pDebugLocation,
                           DWORD dwClsContext, REFIID riid, LPVOID FAR* ppv)
{
  ROUTINE_DBG_MEO00 ("sql38CoCreateInstance");
  return -1;   /* not yet implemented */
};


/*---------------------------------------------------------------------------*/

long sql38CoGetClassObject(CLSID *rclsid, DWORD dwClsContext, 
                           tsp00_Addr pDebugLocation,
                           REFIID riid, LPVOID FAR* ppv)
{
  ROUTINE_DBG_MEO00 ("sql38CoGetClassObject");

  teo78_ClsID       szClsID;
  teo78_Result      rc;
  teo78_ModuleName  ModuleName;
  teo78_ModuleName  OmsLibPath;
  tsp_errtext       errtext;
  HANDLE            hd;
  DLLMAIN           DllMain;
  DLLGETCLASSOBJECT DllGetClassObject;
  char              msgBuffer[256];         

  sprintf(szClsID,"{%8.8X-%4.4X-%4.4X-%2.2X%2.2X-%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X}",
    rclsid->Data1, rclsid->Data2, rclsid->Data3,
    rclsid->Data4[0], rclsid->Data4[1], rclsid->Data4[2],
    rclsid->Data4[3], rclsid->Data4[4], rclsid->Data4[5],
    rclsid->Data4[6], rclsid->Data4[7] );


  tsp00_Pathc szInstPath;
  tsp01_RteError rteError;

  rc = sqlComObjectGetByClsID(szClsID, ModuleName);
  if ( success_eo78 != rc ) 
  {
    switch (rc) 
    {
    case illegal_parameter_eo78:
      sprintf(msgBuffer,"sqlComObjectGetByClsID: illegal_parameter");
      break;
    case access_denied_eo78:
      sprintf(msgBuffer,"sqlComObjectGetByClsID: access_denied_eo78");
      break;
    case entry_not_found_eo78:
      sprintf(msgBuffer,"sqlComObjectGetByClsID: entry_not_found_eo78");
      break;
    default:
      sprintf(msgBuffer,"sqlComObjectGetByClsID with error");     
    }
    MSGCD ((ERR_DCOM_DUMP_INFO, msgBuffer));
    return GEO00D_DCOM_COCLASS_IN_REGISTRY_NOT_FOUND;
  }

  /* only INPROC supported yet */
  if (dwClsContext != CLSCTX_INPROC_SERVER)
  {
    return GEO00D_DCOM_ONLY_INPROC_SUPPORTED;
  }

 	eo38LoadLibraryEx(&ModuleName[0], false, &hd, errtext, sizeof(tsp_errtext));
  if ( hd == 0 ) 
  {
    return GEO00D_DCOM_LIBRARY_NOT_LOADABLE;
  }
  
  DllMain = (DLLMAIN)sqlGetProcAddress(hd, "DllMain", errtext, sizeof(tsp_errtext)-1 );
  if ( NULL == DllMain )
  {
    return GEO00D_DCOM_DLL_MAIN_MISSING;
  }

  if ( ! DllMain(hd, DLL_PROCESS_ATTACH, &ModuleName[0]))
  {
    return GEO00D_DCOM_DLL_MAIN_ERROR;
  }

  DllGetClassObject = (DLLGETCLASSOBJECT)sqlGetProcAddress(hd, 
                            "DllGetClassObject", errtext, sizeof(tsp_errtext)-1 );
  return DllGetClassObject(*rclsid, riid, ppv);
}

/*---------------------------------------------------------------------------*/

externC long sql38CoInitializeEx( void )
{
  return S_OK;
}

/*---------------------------------------------------------------------------*/

externC long sql38CoUninitialize( void )
{
  return S_OK;
}

/*---------------------------------------------------------------------------*/

externC int en38_Eval_Exception ( tsp_dcom_dispatch_parms * parms)
{
  IID loc_iid;
  IUnknown                *lpIUnknown;
  IClassFactory           *lpIFactory;
  CCoClassHash            *HashPtr;
  CHashEntryPtr           currEntry;
  CHashEntryPtr           currSessEntry;
#ifndef USE_KGS
  IliveCacheSink*         pIliveCacheSink;
  char                    msgBuffer[132];
#endif
  parms->hresult = GEO00D_DCOM_DBPROC_CRASHED;
  if (parms->session_context != NULL )
  {
    tsp00_C4          session;
    tsp1_packet*      SQLBuffer_ptr;
    CIliveCacheSink*  pIliveCacheSink;
    ak341GetDebugContext ((tak_all_command_glob*)parms->session_context,
        *REINTERPRET_CAST(tgg91_SessionNo*, &session),
        *REINTERPRET_CAST(tsp00_Addr*, &pIliveCacheSink),
        SQLBuffer_ptr );
    if ( !pIliveCacheSink->CallbackIsActive)
      parms->hresult = -1; // stay alive
  }
#ifdef USE_KGS
  MSGCD ((ERR_DCOM_HRESULT, "eo38VDcomDispatch", parms->hresult ));
#endif
  lpIUnknown = (IUnknown *) parms->idispatch;
  lpIFactory = (IClassFactory *) parms->ifactory;
  
  WAIT_UNTIL_ALONE (DCOM_EXCLUSIVE);
  HashPtr = REINTERPRET_CAST(CCoClassHash*,(DCOM_HEAD));
  if ( NULL != HashPtr ) 
  {
    /* Release session related debugger                        */
    HashPtr->ResetEntrySearch( 0, SearchALL_eeo38 );
    currEntry = HashPtr->NextEntry();
    while ( NULL != currEntry)
    {
      if ((currEntry->getIUnknown() == lpIUnknown) ||
        (currEntry->getIUnknown() == lpIFactory))
      {
        memcpy (&loc_iid.Data1, currEntry->getGuid(), sizeof (loc_iid));
        if (currEntry->getIUnknown() == lpIUnknown)
        {
#ifdef USE_KGS
          MSGCD ((ERR_DCOM_COMETHOD_CRASHED,
            loc_iid.Data1, loc_iid.Data2, loc_iid.Data3,
            loc_iid.Data4[0], loc_iid.Data4[1], loc_iid.Data4[2],
            loc_iid.Data4[3], loc_iid.Data4[4], loc_iid.Data4[5],
            loc_iid.Data4[6], loc_iid.Data4[7], parms->dispid));
#else
          sprintf(&msgBuffer[0],"COM-METHOD CRASH! IID:{%8.8X-%4.4X-%4.4X-%2.2X%2.2X-%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X} FKTNO:%d",
            loc_iid.Data1, loc_iid.Data2, loc_iid.Data3,
            loc_iid.Data4[0], loc_iid.Data4[1], loc_iid.Data4[2],
            loc_iid.Data4[3], loc_iid.Data4[4], loc_iid.Data4[5],
            loc_iid.Data4[6], loc_iid.Data4[7], parms->dispid);
          memcpy( &pIliveCacheSink, &parms->session_context, sizeof(pIliveCacheSink));
          pIliveCacheSink->ConsoleMsg (strlen(&msgBuffer[0]), (unsigned char*)&msgBuffer[0] );
#endif
        }
        else
        {
#ifdef USE_KGS
          MSGCD ((ERR_DCOM_DBPROC_CRASHED,
            loc_iid.Data1, loc_iid.Data2, loc_iid.Data3,
            loc_iid.Data4[0], loc_iid.Data4[1], loc_iid.Data4[2],
            loc_iid.Data4[3], loc_iid.Data4[4], loc_iid.Data4[5],
            loc_iid.Data4[6], loc_iid.Data4[7], parms->dispid));
#else
          sprintf(&msgBuffer[0],"COM-DBPROC CRASH! IID:{%8.8X-%4.4X-%4.4X-%2.2X%2.2X-%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X} FKTNO:%d",
            loc_iid.Data1, loc_iid.Data2, loc_iid.Data3,
            loc_iid.Data4[0], loc_iid.Data4[1], loc_iid.Data4[2],
            loc_iid.Data4[3], loc_iid.Data4[4], loc_iid.Data4[5],
            loc_iid.Data4[6], loc_iid.Data4[7], parms->dispid);
          memcpy( &pIliveCacheSink, &parms->session_context, sizeof(pIliveCacheSink));
          pIliveCacheSink->ConsoleMsg (strlen(&msgBuffer[0]), (unsigned char*)&msgBuffer[0] );
#endif
        }
        break;
      }
      currEntry = HashPtr->NextEntry(); 
    }
  }
  CLEARLOCK (DCOM_EXCLUSIVE);
  return 0;
}

/*---------------------------------------------------------------------------*/
/***
externC BOOL sql38LoadLibOms(tsp_errtext* errtext)
{
  teo78_ModuleName  OmsLibPath;
  tsp00_Pathc       szInstPath;
  tsp01_RteError    rteError;
  HANDLE            hd;
  if ( sqlGetDbrootLibPath (szInstPath, TERM_WITHOUT_DELIMITER_EO01, &rteError ) ) 
  {
    strcpy(OmsLibPath, szInstPath);
    strcat(OmsLibPath, "/liboms" );
    hd = sqlLoadLibrary(OmsLibPath, *errtext, sizeof(tsp_errtext)-1);
  }
  return (hd != 0);
}
***/

==> sapdb-source-7.4.03.32.tgz: V74_03_32/SAPDB_ORG/sys/src/en/ven38.cpp <==
/* @lastChanged: "1998-10-06  15:03"
 * @filename:    ven38.cpp
 * @purpose:     "COM_Interface"
 * @release:     7.2.0.0
 * @see:         ""
 *
 * @copyright:   "Copyright by SAP AG, 1998"


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

 */

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#if defined(NMP)
#define RWSTD_NO_NAMESPACE  1
#include <sys/user.h>
#endif

#include "gsp00.h"
#include "heo01.h"
#include "heo02x.h"
#include "heo78.h"
#include "heo00.h" //
#include "gco00.h"
#include "geo00d.h"
#include "hak341.h"
#include "geo38.h"
#include "hen38.h"

#include "heo52.h"
#include "vsp002c.h"
#include "heo46.h"
#include "geo60.h"
#include "heo60.h"
#include "geo50_0.h"
#include "gen500.h"
#include "geo60.h"
#include "heo38.h"
#include "geo573.h" // PTS 1112369

 /*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

#ifdef NO_INLINES
# define _INLINE
#else
# define _INLINE	inline
#endif

#define DCOM_EXCLUSIVE  KGS->DCOM_MEM.lExclusive
#define DCOM_HEAD       KGS->DCOM_MEM.lphead
#define DCOM_FREE       KGS->DCOM_MEM.lpfree

typedef BOOL (*DLLMAIN) (HANDLE hInstance,
                         DWORD  dwReason, 
                         char*  szModule);

typedef HRESULT (*DLLGETCLASSOBJECT) (REFCLSID rclsid, REFIID riid, LPVOID* ppv);

/*---------------------------------------------------------------------------*/

long sql38CoCreateInstance(CLSID *rclsid, REFIID riid, LPVOID FAR* ppv)
{
  ROUTINE_DBG_MEO00 ("sql38CoCreateInstance");
  return -1;   /* not yet implemented */
};


/*---------------------------------------------------------------------------*/

long sql38CoGetClassObject(CLSID *rclsid, REFIID riid, LPVOID FAR* ppv)
{
  ROUTINE_DBG_MEO00 ("sql38CoGetClassObject");

  teo78_ClsID       szClsID;
  teo78_Result      rc;
  teo78_ModuleName  ModuleName;
  teo78_ModuleName  OmsLibPath;
  tsp_errtext       errtext;
  HANDLE            hd;
  DLLMAIN           DllMain;
  DLLGETCLASSOBJECT DllGetClassObject;
  char              msgBuffer[256];         

  sprintf(szClsID,"{%8.8X-%4.4X-%4.4X-%2.2X%2.2X-%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X}",
    rclsid->Data1, rclsid->Data2, rclsid->Data3,
    rclsid->Data4[0], rclsid->Data4[1], rclsid->Data4[2],
    rclsid->Data4[3], rclsid->Data4[4], rclsid->Data4[5],
    rclsid->Data4[6], rclsid->Data4[7] );


  tsp00_Pathc szInstPath;
  tsp01_RteError rteError;

  rc = sqlComObjectGetByClsID(szClsID, ModuleName);
  if ( success_eo78 != rc ) 
  {
    switch (rc) 
    {
    case illegal_parameter_eo78:
      sprintf(msgBuffer,"sqlComObjectGetByClsID: illegal_parameter");
      break;
    case access_denied_eo78:
      sprintf(msgBuffer,"sqlComObjectGetByClsID: access_denied_eo78");
      break;
    case entry_not_found_eo78:
      sprintf(msgBuffer,"sqlComObjectGetByClsID: entry_not_found_eo78");
      break;
    default:
      sprintf(msgBuffer,"sqlComObjectGetByClsID with error");     
    }
    MSGD ((ERR_DCOM_DUMP_INFO, msgBuffer));
    return GEO00D_DCOM_COCLASS_IN_REGISTRY_NOT_FOUND;
  }

 	eo38LoadLibraryEx(&ModuleName[0], false, &hd, errtext, sizeof(tsp_errtext));
  if ( hd == 0 ) 
  {
    return GEO00D_DCOM_LIBRARY_NOT_LOADABLE;
  }
  
  DllMain = (DLLMAIN)sqlGetProcAddress(hd, "DllMain", errtext, sizeof(tsp_errtext)-1 );
  if ( NULL == DllMain )
  {
    return GEO00D_DCOM_DLL_MAIN_MISSING;
  }

  if ( ! DllMain(hd, DLL_PROCESS_ATTACH, &ModuleName[0]))
  {
    return GEO00D_DCOM_DLL_MAIN_ERROR;
  }

  DllGetClassObject = (DLLGETCLASSOBJECT)sqlGetProcAddress(hd, 
                            "DllGetClassObject", errtext, sizeof(tsp_errtext)-1 );
  if (DllGetClassObject == NULL) {
    return GEO00D_DCOM_NO_COCLASS_OBJECT_FOUND;
  }
  //return DllGetClassObject(*rclsid, riid, ppv);   // PTS 1112369
  long hr;                                          // PTS 1112369
  try {                                             // PTS 1112369
    hr = DllGetClassObject(*rclsid, riid, ppv);
  }                                                 // PTS 1112369
  catch ( BAD_ALLOC_GEO573 ) {                      // PTS 1112369
    hr = GEO00D_DCOM_BAD_ALLOC;                     // PTS 1112369
  }                                                 // PTS 1112369
  return hr;                                        // PTS 1112369
}

/*---------------------------------------------------------------------------*/

externC int en38_Eval_Exception ( tsp_dcom_dispatch_parms * parms)
{
  IID loc_iid;
  IUnknown                *lpIUnknown;
  IClassFactory           *lpIFactory;
  CCoClassHash            *HashPtr;
  CHashEntryPtr           currEntry;
  CHashEntryPtr           currSessEntry;
  IliveCacheSink*         pIliveCacheSink;
  char                    msgBuffer[132];
  parms->hresult = GEO00D_DCOM_DBPROC_CRASHED;
  if (parms->session_context != NULL )
  {
    tsp00_C4          session;
    tsp1_packet*      SQLBuffer_ptr;
    CIliveCacheSink*  pIliveCacheSink;
    ak341GetDebugContext ((tak_all_command_glob*)parms->session_context,
        *REINTERPRET_CAST(tgg91_SessionNo*, &session),
        *REINTERPRET_CAST(tsp00_Addr*, &pIliveCacheSink),
        SQLBuffer_ptr );
    if ( !pIliveCacheSink->CallbackIsActive)
      parms->hresult = -1; // stay alive
  }
  MSGD ((ERR_DCOM_HRESULT, "eo38VDcomDispatch", parms->hresult ));
  lpIUnknown = (IUnknown *) parms->idispatch;
  lpIFactory = (IClassFactory *) parms->ifactory;
  
  WAIT_UNTIL_ALONE (DCOM_EXCLUSIVE);
  HashPtr = REINTERPRET_CAST(CCoClassHash*,(DCOM_HEAD));
  if ( NULL != HashPtr ) 
  {
    /* Release session related debugger                        */
    HashPtr->ResetEntrySearch( 0, SearchALL_eeo38 );
    currEntry = HashPtr->NextEntry();
    while ( NULL != currEntry)
    {
      if ((currEntry->getIUnknown() == lpIUnknown) ||
        (currEntry->getIUnknown() == lpIFactory))
      {
        memcpy (&loc_iid.Data1, currEntry->getGuid(), sizeof (loc_iid));
        if (currEntry->getIUnknown() == lpIUnknown)
        {
          MSGD ((ERR_DCOM_COMETHOD_CRASHED,
            loc_iid.Data1, loc_iid.Data2, loc_iid.Data3,
            loc_iid.Data4[0], loc_iid.Data4[1], loc_iid.Data4[2],
            loc_iid.Data4[3], loc_iid.Data4[4], loc_iid.Data4[5],
            loc_iid.Data4[6], loc_iid.Data4[7], parms->dispid));
        }
        else
        {
          MSGD ((ERR_DCOM_DBPROC_CRASHED,
            loc_iid.Data1, loc_iid.Data2, loc_iid.Data3,
            loc_iid.Data4[0], loc_iid.Data4[1], loc_iid.Data4[2],
            loc_iid.Data4[3], loc_iid.Data4[4], loc_iid.Data4[5],
            loc_iid.Data4[6], loc_iid.Data4[7], parms->dispid));
        }
        break;
      }
      currEntry = HashPtr->NextEntry(); 
    }
  }
  CLEARLOCK (DCOM_EXCLUSIVE);
  return 0;
}

/*---------------------------------------------------------------------------*/
/***
externC BOOL sql38LoadLibOms(tsp_errtext* errtext)
{
  teo78_ModuleName  OmsLibPath;
  tsp00_Pathc       szInstPath;
  tsp01_RteError    rteError;
  HANDLE            hd;
  if ( sqlGetDbrootLibPath (szInstPath, TERM_WITHOUT_DELIMITER_EO01, &rteError ) ) 
  {
    strcpy(OmsLibPath, szInstPath);
    strcat(OmsLibPath, "/liboms" );
    hd = sqlLoadLibrary(OmsLibPath, *errtext, sizeof(tsp_errtext)-1);
  }
  return (hd != 0);
}
***/
